/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// stateless component factories that are probably safe
//

#include "fggl/entity/module.hpp"
#include "fggl/math/types.hpp"

namespace fggl::entity {

	void make_transform(const entity::ComponentSpec& spec, EntityManager& manager, const entity::EntityID entity){
		manager.add<math::Transform>(entity);
	}

	void install_component_factories(entity::EntityFactory* factory) {
		factory->bind(math::Transform::guid, make_transform);
	}

	bool entity_svc_factory(modules::ModuleService service, modules::Services& services) {
		if ( service == EntityFactory::service) {
			auto* factory = services.create<EntityFactory>();
			install_component_factories(factory);

			// we are responsible for prefabs...
			auto* assetLoader = services.get<assets::Loader>();
			assetLoader->setFactory(PROTOTYPE_ASSET, [factory](const assets::AssetGUID& a, assets::AssetData b) {
				return load_prototype(factory, a, b); }, assets::LoadType::PATH );

			return true;
		}
		return false;
	}
	const modules::ServiceFactory ECS::factory = entity_svc_factory;
}